#!/usr/bin/python
'''
LTE Daemon with autoswitch feature
    Created on 3 Jun 2019 - version 0.10
@author: martyy <triska@faster.cz>
'''
import os, sys, stat
import logging, logging.handlers
import re
import time
import serial
import signal
import ConfigParser
import argparse
import exceptions
import fcntl
import struct
import random
import subprocess
from time import sleep


# Default values, can be overriden in the .conf file
DEFAULT_LOGGING_LEVEL = "INFO"
DEFAULT_LOG_PATH = None  # Log path is none -> use syslog instead of an user log file
DEFAULT_LOGFILE_SIZE = 131072  # 128 kiB
RESCUE_THRESHOLD = 350 # Not-success ping to modem restart

# Global definitions
DAEMON_VERSION = "0.9"
MODEM_PORT = "/dev/ttyLTE_MNG"  # Symlink to ttyACM0
CONFILE_PATH = "/etc/unipi/lte.conf"
pidfile = "/run/unipilte.pid"
MODEM_RESTART_PIN = 18
NIC_CONF_PATH = "/etc/network/interfaces.d/"
USB_PORT = "3-1:1.0"

# Indicators moved to sysleds (requires unipi-kernel-modules or neuron-kernel version >= 1.42)
# And appropriate firmware (version >=5.50)
LED_SIM_PATH = "/sys/class/leds/unipi:green:sysled-x0"
LED_NET_PATH = "/sys/class/leds/unipi:green:sysled-x1"
LED_SIG_PATH = "/sys/class/leds/unipi:green:sysled-x2"
LED_INT_PATH = "/sys/class/leds/unipi:green:sysled-x3"

# Network states
WAN_STATE_INIT = 0
WAN_STATE_LTE_ON = 1
WAN_STATE_LTE_OFF = 2
WAN_STATE_ETH_ON = 3
WAN_STATE_LTE_ON_ETH_OFF = 4
WAN_STATE_LTE_OFF_ETH_ON = 5
WAN_STATE_LTE_ON_ETH_ON = 6
WAN_STATE_LTE_OFF_ETH_OFF = 7
WAN_STATE_ETH_OFF = 8

# Specific error codes (no all known errors listed)
ERROR_CODE_UNSPEC = 0
ERROR_CODE_NO_SIM = 1
ERROR_CODE_SIM_ERROR = 2
ERROR_CODE_WRONG_PIN = 3

# Actions usually performed in non-standard cases
ACTION_NOTHING = 0
ACTION_CONTINUE = 1
ACTION_REINIT = 2
ACTION_SOFTRESTART = 3
ACTION_HARDRESTART = 4
ACTION_SUSPEND = 5
ACTION_HALT = 6

# Inherited exceptions
class ModemCreateInstanceException(Exception):
    pass


class TimeoutException(Exception):
    pass

class ConnectionError(Exception):
    pass

class ModemErrorException(Exception):
    pass


class LEDErrorException(Exception):
    pass


class NewSettinngs(Exception):
    pass


# Global variables
current_config = {}
logger = None


# Graceful exit with log event generated
def sig_handler(signum, frame):

    logger = logging.getLogger('unipilte')
    logger.info("Exiting after received signal {}".format(signum))
    led = LedServer()
    led.all_off()
    os.unlink(pidfile)
    os._exit(0)

# Some of the network providers can assign WAN IP from the local range(s).
# In the case of IP conflict on both subnets, changes the appropriate IPs.
def resolve_subnets(ip_wan, ip_private, subnet):

    v1_ip = "192.168.101.1"  # The first variant
    v1_dhcp = "192.168.101.100"  # The first variant's DHCP range
    v2_ip = "10.11.12.1"  # Second variant
    v2_dhcp = "10.11.12.100"  # Second variant's DHCP range
    new_subnet = "255.255.255.0"

    def ip_to_binary(ip):  # Helper function converting string representation of given ip to binary integer
        octet_list_int = ip.split(".")
        octet_list_bin = [format(int(i), '08b') for i in octet_list_int]
        binary = ("").join(octet_list_bin)
        return binary

    subnet_wan = int(ip_to_binary(ip_wan), 2) & int(ip_to_binary(subnet), 2)  # Bitwise AND of IP and SUBNET
    subnet_private = int(ip_to_binary(ip_private), 2) & int(ip_to_binary(subnet), 2)  # Bitwise AND of IP and SUBNET

    logger.debug("WAN IP {}, Private IP {}; {} ".format(ip_wan, ip_private, "CONFLICT" if subnet_private == subnet_wan else "NON-CONFLICT"))

    if subnet_wan == subnet_private:  # According to the doc from UBlox, ip range of LAN and WAN must be different
        logger.info("Resolving IP conflict")
        if ip_private == v1_ip:
            return {'ip':v2_ip, 'dhcp':v2_dhcp, 'mask':new_subnet}
        else:
            return {'ip':v1_ip, 'dhcp':v1_dhcp, 'mask':new_subnet}

    return None

# Checks if config file (typically on path /etc/network/interfaces.d/) exists. If no, creates it.
def check_iface_config_file(iface):
    global NIC_CONF_PATH

    if not os.path.isfile(NIC_CONF_PATH + iface):
        file = open(NIC_CONF_PATH + iface, 'w+')
        file.write("auto {0}\niface {0} inet dhcp".format(iface))
        file.close()
        logger.info("Created {} iface file".format(iface))
        ConnectionGuard.try_interface_up(iface)
    else:
        logger.debug("Interface {} already exists.".format(iface))


# Get LTE NIC name and creates appropriate config file if necessary
def get_lte_iface_name(create_config_file=True):
    iface = None
    ifaces_path = '/sys/class/net'

    def look_for_nic():  # Helper function iterating over directory and looking for NIC on desired USB port (e.g. returns eth1 or wwan0)
        for filename in os.listdir(ifaces_path):
            path = os.readlink(ifaces_path + "/" + filename)
            if USB_PORT in path:
                return filename
        return None

    for i in range(3):
        iface = look_for_nic()
        if iface:
            break
        sleep(3)

    if not iface:
        return None

    if create_config_file:
        check_iface_config_file(iface)

    return iface


# Delete file previously created by check_iface_config_file. Used in purge/remove process.
def remove_iface_confile():
    iface_name = get_lte_iface_name(False)
    if iface_name is None:
        print("No LTE interface found, nothing to remove...")
        return
    iface_path = NIC_CONF_PATH + iface_name
    if os.path.exists(iface_path):
        os.system("ifdown --force " + iface_name)
        print("Removing network interface " + iface_name)
        os.remove(iface_path)
    else:
        print("No interface config found, nothing to remove...")


# Parse of config file with int/bool data types conversion and default values
def getFromConfile(config, section, option, default, boolean=False, integer=False):
    global logger
    try:
        value = config.get(section, option)
        if boolean == True :
            value = value.lower() in ("yes", "true", "t", "1")
        elif integer == True :
            value = int(value)
    except ValueError:
        if logger is not None:
            logger.warning("Cannot parse integer parameter {}:{}, using default value: {}".format(section, option, default))
        return default
    except:
        if logger is not None:
            logger.debug("Cannot parse parameter {}:{}, using default value: {}".format(section, option, default))
        return default

    return value


# Collects and computes statistical data related to cellular network quality and WAN network availability
class Stats(object):

    def __init__(self, period=120, ping_active=False):
        self.period = period
        self.reset()
        self.ping_active = ping_active

    def report_partial(self, rssi, sigqual):  # Commit often read values only, called from main loop

        try:
            rssi = int(rssi)
            sigqual = int(rssi)
        except:
            return

        if rssi == 99: #Not valid value
            return

        self.cnt = self.cnt + 1
        if self.rssi_max < rssi: self.rssi_max = rssi
        elif self.rssi_min > rssi: self.rssi_min = rssi
        self.rssi_sum += rssi

        self.sigqual = sigqual

    def report_metrics(self, rssi, sigqual, wan_status, nettype, ping=None):  # Commit all values, called form main loop
        if not self.ping_active: return
        self.report_partial(rssi, sigqual)
        if wan_status == WAN_STATE_LTE_OFF or wan_status == WAN_STATE_LTE_OFF_ETH_OFF: self.wan_outage_cnt = self.wan_outage_cnt + 1
        elif wan_status == WAN_STATE_LTE_ON_ETH_OFF : self.backup_cnt = self.backup_cnt + 1
        self.nettypes.add(nettype)

    def reset(self):
        self.cnt = 0
        self.rssi_sum = 0
        self.rssi_min = 100
        self.rssi_max = 0
        self.wan_outage_cnt = 0
        self.backup_cnt = 0
        self.last_summary = time.time()
        self.sigqual = 0
        self.nettypes = set()

    def get_summmary(self):  # Get report output in text form (e.g. for periodical storing to a log file)

        if self.ping_active :
            sum_str = "RSSI: (MIN {}, MAX {}, AVG {}), SIGQUAL: {}, NET-TYPES: {}, OFFLINE_CNT: {}, BACKUP_CNT: {}".format(
                self.rssi_min,
                self.rssi_max,
                self.rssi_sum / self.cnt,
                self.sigqual,
                list(self.nettypes),
                self.wan_outage_cnt,
                self.backup_cnt)
        else:
                sum_str = "RSSI: (MIN {}, MAX {}, AVG {}) SIGQUAL: {} NET-TYPES: {} ".format(
                self.rssi_min,
                self.rssi_max,
                self.rssi_sum / self.cnt,
                self.sigqual,
                list(self.nettypes))

        return sum_str

    def is_time_fullfilled(self):  # Returns true if it is time to generate stats into the log file
        if not self.period:
            return False
        return (self.last_summary + self.period) < time.time()


# Provides ping and autoswitch functionality
class ConnectionGuard(object):

    GW_MASK_LTE = 1
    GW_MASK_ETH = 2

    def isOnline(self):  # True if the unit has access to the WAN network (found by ping)

        if self.wan_state not in (WAN_STATE_LTE_OFF, WAN_STATE_LTE_OFF_ETH_OFF, WAN_STATE_INIT):
            return False
        return True

    def __init__(self):

        global current_config
        self.local_lte_ip = None
        self.current_eth_metric = 0
        self.current_lte_metric = 0
        self.gw_bitmask = 0
        self.rtable_valid = True
        self.reset_state()

    def reset_state(self):
        self.wan_state = WAN_STATE_INIT
        self.fail_counterA = 0
        self.fail_counterB = 0
        self.rescue_counter = -1

    # Detects if the main interface is not available (e.g. cable is off)
    # Reads and compares metrics of the intefaces (both main and backup)
    def __read_default_gateway(self):
        global current_config

        self.current_eth_metric = -1
        self.current_lte_metric = -1
        gw_bitmask = 0
        routecount = 0

        STATE_PATH = "/sys/class/net/{}/operstate"
        upstrings = ("up", "unknown")

        try:
            with file(STATE_PATH.format(current_config['ping']['eth_iface']), "r") as f:
                res = f.read()
                if any (x in res for x in upstrings):
                    gw_bitmask |= self.GW_MASK_ETH
        except:
            pass

        try:
            with file(STATE_PATH.format(current_config['lte_iface']), "r") as f:
                res = f.read()
                if any (x in res for x in upstrings):
                    gw_bitmask |= self.GW_MASK_LTE
                elif current_config['lte_autokeep_on'] is True:
                    logger.warn("UP {} LTE interface that has been put DOWN".format(current_config['lte_iface']))
                    self.try_interface_up(current_config['lte_iface'], down_firstly=True)
        except:
            pass

        with open("/proc/net/route") as fh:
            for line in fh:
                routecount = routecount + 1
                fields = line.strip().split()  # Default gateway ha
                if (fields[1] == '00000000') and (fields[0] == current_config['ping']['eth_iface']):
                    self.current_eth_metric = int(fields[6])
                elif (fields[1] == '00000000') and (fields[0] == current_config['lte_iface']):
                    self.current_lte_metric = int(fields[6])

                    if self.local_lte_ip is None:
                        loc_ip = struct.pack("<L", int(fields[2], 16))
                        self.local_lte_ip = ".".join(str(ord(c)) for c in loc_ip)
                        if self.local_lte_ip == "0.0.0.0":
                            self.local_lte_ip = None
                        # logger.debug("GW IP {}".format(self.local_lte_ip))

        is_rtable_valid = (True if (routecount > 2) else False)
        # is_current_iface_main = (True if ((main_metric < backup_metric) and (gw_bitmask & 1)) else False)

        logger.debug("Current iface metrics main_iface:{} backup_iface:{} Local LTE ip {}, routing items {}, gw_bitmask {}".format(self.current_eth_metric,
                                                                                                                     self.current_lte_metric, self.local_lte_ip,
                                                                                                                     routecount, gw_bitmask))


        return (gw_bitmask, is_rtable_valid)

    @staticmethod
    def try_interface_up(iface, down_firstly = False):
        if down_firstly is True:
            subprocess.call("ifdown --force " + iface)
        for i in range(3): # Wait max 18 sec for DHCP IP address assignment
            subprocess.call("ip link set dev {} up".format(iface), shell=True)
            subprocess.call("ifup " + iface, shell=True)
            for j in range (6):
                ret = subprocess.check_output("ip addr show " + iface, shell=True)
                if "inet " in ret:
                    return
                time.sleep(1)

        logger.error("Cannot obtain IP address from LTEs DHCP server")


    def __update_resolver_file(self, lte_as_dns):
        global current_config

        # Check IP validity

        FILEPATH = "/etc/resolvconf/resolv.conf.d/head"
        LINE = "nameserver {}\n".format(self.local_lte_ip) # Toto vznikne
        data = None
        changed = False

        try: # If the config file does not exists
            with open(FILEPATH, "r") as f:
                data = f.read()
        except Exception as e:
            logger.warn("Cannot change DNS resolver file: " + str(e))
            return

        if LINE in data:
            newdata = re.sub(r"\#?({})".format(LINE), r"{}\1".format("" if lte_as_dns is True else "#"), data)
            if newdata != data:
                with open(FILEPATH, "w") as f:
                    f.write(newdata)
                changed = True

        elif lte_as_dns == True:
            MSG = "#This line has been added by unipi-lte daemon\n{}".format(LINE)
            with open(FILEPATH, "a") as f:
                f.write(MSG)
            changed = True

        if changed is not False:
            status = subprocess.call("resolvconf " + "-u", shell=True)


    def __switch_default_iface(self, toBackup):  # Switches network ifaces (change their metrics)
        global current_config

        new_metric_shift = 0
        # DEF Change metric - lokalni funkce

        if toBackup and (self.gw_bitmask & self.GW_MASK_LTE):

            logger.debug("Switching to LTE interface {}".format(current_config['lte_iface']))

            if not (self.gw_bitmask & self.GW_MASK_LTE):
                return

            if self.current_lte_metric + 1 == self.current_eth_metric:
                new_metric_shift = 1

            new_metric = self.current_eth_metric - 1 - new_metric_shift

            logger.debug("ip route add default via {} dev {} metric {}".format(self.local_lte_ip, current_config['lte_iface'], new_metric))
            logger.debug("ip route delete default dev {} metric {}".format(current_config['lte_iface'], self.current_lte_metric))

            os.system("ip route add default via {} dev {} metric {}".format(self.local_lte_ip, current_config['lte_iface'], new_metric))
            os.system("ip route delete default dev {} metric {}".format(current_config['lte_iface'], self.current_lte_metric))

            self.current_lte_metric = new_metric

            try:
                self.__update_resolver_file(lte_as_dns = True)
            except Exception as E:
                logger.critical("Failure: " + str(E))

        elif self.gw_bitmask & self.GW_MASK_ETH:

            logger.debug("Switching to main interface {}".format(current_config['ping']['eth_iface']))

            if self.current_lte_metric - 1 == self.current_eth_metric:
                new_metric_shift = 1

            new_metric = self.current_eth_metric + 1 + new_metric_shift

            logger.debug("ip route add default via {} dev {} metric {}".format(self.local_lte_ip, current_config['lte_iface'], new_metric))
            logger.debug("ip route delete default dev {} metric {}".format(current_config['lte_iface'], self.current_lte_metric))
            os.system("ip route add default via {} dev {} metric {}".format(self.local_lte_ip, current_config['lte_iface'], new_metric))
            os.system("ip route delete default dev {} metric {}".format(current_config['lte_iface'], self.current_lte_metric))

            self.current_lte_metric = new_metric

            try:
                self.__update_resolver_file(lte_as_dns = False)
            except Exception as E:
                logger.critical("Failure: " + str(E))

            #self.__update_resolver_file(lte_as_dns = False)

    def get_wan_status(self):  # Get current status of the connection (see WAN_STATE_xxxx constants)
        return self.wan_state

    def get_rtable_valid(self):  # Returns true if routing table contains at least one valid record
        return self.rtable_valid

    def __autoswitch(self, lte_status):  # Autoswitch core called from ping function (if enabled in the config file)

        global current_config

        LTE_UNCHANGED = 0
        LTE_OK = 1
        LTE_OFFLINE = 2

        ETH_UNCAHNGED = 0
        ETH_GOES_OFFLINE = 1
        ETH_GOES_ONLINE = 2
        eth_status = ETH_UNCAHNGED

        response_b = 1  # Ping failed as default

        self.gw_bitmask, self.rtable_valid = self.__read_default_gateway()

        if not self.rtable_valid:  # Do not perform autoswitch if routing table is not valid (typically after startup)
            logger.debug("Routing table is not (yet) valid, autoswitch disabled")
            return False

        if self.gw_bitmask & self.GW_MASK_ETH :  # If main iface is present, ping it
            response_b = os.system("ping -c 1 -W {0} -I {1} -s {2} {3} >/dev/null".format(
                current_config['ping']['timeout'],
                current_config['ping']['eth_iface'],
                "32", current_config['ping']['ip_eth']))
        elif self.wan_state in {WAN_STATE_LTE_OFF_ETH_ON, WAN_STATE_LTE_ON_ETH_ON, WAN_STATE_INIT}:  # If main interface has been disconnected, switch to backup immediately
            self.fail_counterB = current_config['ping']['threshold'] + 1
            logger.info("Main GW is not present ! Switching to backup iface {} immediately...".format(current_config['lte_iface']))

        if response_b and self.wan_state not in {WAN_STATE_ETH_OFF, WAN_STATE_LTE_OFF_ETH_OFF, WAN_STATE_LTE_ON_ETH_OFF}:  # Ping on main interface failed and ETH is ON

            if self.fail_counterB >= current_config['ping']['threshold']:  # If counter overflows, ETH ON -> ETH OFF
                eth_status = ETH_GOES_OFFLINE
                self.__switch_default_iface(toBackup=True)  # Switch to backup
                self.gw_bitmask, self.rtable_valid = self.__read_default_gateway() # Refresh GW info
                if self.fail_counterB == current_config['ping']['threshold']:
                    logger.info("Ping failure threshold reached {} on main {}.Switching to backup {}...".format(
                        current_config['ping']['threshold'],
                        current_config['ping']['eth_iface'],
                        current_config['lte_iface']))

            else:
                self.fail_counterB = self.fail_counterB + 1

            logger.debug("Ping failed on {} no. {}; Threshold: {}".format(
                current_config['ping']['eth_iface'],
                self.fail_counterB, current_config['ping']['threshold']))

        elif (not response_b) and self.wan_state not in {WAN_STATE_ETH_ON, WAN_STATE_LTE_OFF_ETH_ON, WAN_STATE_LTE_ON_ETH_ON}:  # Ping OK, switch back to main iface

            if self.wan_state != WAN_STATE_INIT:
                logger.info("Main interface {} becomes available, switching to it".format(
                    current_config['ping']['eth_iface']))
                self.__switch_default_iface(toBackup=False)  # Switch to main
                self.gw_bitmask, self.rtable_valid = self.__read_default_gateway() # Refresh GW info

            eth_status = ETH_GOES_ONLINE
            logger.debug("ETH Ping OK")

        else:
            if not response_b:
                logger.debug("ETH Ping OK")
            self.fail_counterB = 0

        for i in range(2):  # Resolve current state variable
            if eth_status == ETH_GOES_ONLINE:
                if self.wan_state in (WAN_STATE_INIT, WAN_STATE_ETH_OFF):
                    self.wan_state = WAN_STATE_ETH_ON
                elif self.wan_state in (WAN_STATE_LTE_ON, WAN_STATE_LTE_ON_ETH_OFF):
                    self.wan_state = WAN_STATE_LTE_ON_ETH_ON
                elif self.wan_state in (WAN_STATE_LTE_OFF, WAN_STATE_LTE_OFF_ETH_OFF):
                    self.wan_state = WAN_STATE_LTE_OFF_ETH_ON

            elif eth_status == ETH_GOES_OFFLINE:
                if self.wan_state in (WAN_STATE_INIT, WAN_STATE_ETH_ON):
                    self.wan_state = WAN_STATE_ETH_OFF
                elif self.wan_state in (WAN_STATE_LTE_ON, WAN_STATE_LTE_ON_ETH_ON):
                    self.wan_state = WAN_STATE_LTE_ON_ETH_OFF
                elif self.wan_state in (WAN_STATE_LTE_OFF, WAN_STATE_LTE_OFF_ETH_ON):
                    self.wan_state = WAN_STATE_LTE_OFF_ETH_OFF

            if lte_status == LTE_OK:
                if self.wan_state in (WAN_STATE_INIT, WAN_STATE_LTE_OFF):
                    self.wan_state = WAN_STATE_LTE_ON
                elif self.wan_state in (WAN_STATE_ETH_ON, WAN_STATE_LTE_OFF_ETH_ON):
                    self.wan_state = WAN_STATE_LTE_ON_ETH_ON
                elif self.wan_state in (WAN_STATE_ETH_OFF, WAN_STATE_LTE_OFF_ETH_OFF):
                    self.wan_state = WAN_STATE_LTE_ON_ETH_OFF

            elif lte_status == LTE_OFFLINE:
                if self.wan_state in (WAN_STATE_INIT, WAN_STATE_LTE_ON):
                    self.wan_state = WAN_STATE_LTE_OFF
                elif self.wan_state in (WAN_STATE_ETH_ON, WAN_STATE_LTE_ON_ETH_ON):
                    self.wan_state = WAN_STATE_LTE_OFF_ETH_ON
                elif self.wan_state in (WAN_STATE_ETH_OFF, WAN_STATE_LTE_ON_ETH_OFF):
                    self.wan_state = WAN_STATE_LTE_OFF_ETH_OFF

        if self.gw_bitmask == (self.GW_MASK_ETH | self.GW_MASK_LTE):

            if self.current_lte_metric == -1 and self.local_lte_ip is not None: # We have to wait for IP address obtaining via DHCP
                logger.warning("Backup connection {} default route is missing, re-setting...".format(current_config['lte_iface']))
                self.__switch_default_iface(toBackup=True)

            elif (self.wan_state == WAN_STATE_LTE_ON_ETH_OFF) and (self.current_lte_metric >= self.current_eth_metric):
                logger.warning("Routing table metric mismatch A (changed by other application?), re-setting... {} {}".format(self.current_lte_metric, self.current_eth_metric))
                self.__switch_default_iface(toBackup=True)

            elif (self.wan_state in (WAN_STATE_LTE_OFF_ETH_ON, WAN_STATE_LTE_ON_ETH_ON)) and (self.current_eth_metric >= self.current_lte_metric):
                logger.warning("Routing table metric mismatch B (changed by other application?), re-setting...")
                self.__switch_default_iface(toBackup=False)


        return True
        # return not (bool(response_main and response_backup))

    def ping(self):  # Checks the WAN connection, set state varaibles(s) and handle autoswitch stuff if enabled
        global current_config

        LTE_UNCHANGED = 0
        LTE_OK = 1
        LTE_OFFLINE = 2
        lte_status = LTE_UNCHANGED

        logger.debug("ping -c 1 -W {0} -I {1} -s {2} {3} >/dev/null".format(current_config['ping']['timeout'], current_config['lte_iface'], "32", current_config['ping']['ip_lte']))
        # Ping on the backup iface (LTE)
        response_lte = os.system("ping -c 1 -W {0} -I {1} -s {2} {3} >/dev/null".format(
            current_config['ping']['timeout'],
            current_config['lte_iface'],
            "32",
            current_config['ping']['ip_lte']))

        if response_lte :  # Ping not successful
            if self.rescue_counter >= 0:
                if self.rescue_counter > RESCUE_THRESHOLD:
                    self.rescue_counter = 0
                    raise ConnectionError

                self.rescue_counter = self.rescue_counter + 1

            logger.debug("Ping failed GSM no. {}; Threshold: {}".format(self.fail_counterA, current_config['ping']['threshold']))
            if self.fail_counterA >= current_config['ping']['threshold']:

                if self.wan_state in {WAN_STATE_INIT, WAN_STATE_ETH_ON}:
                    logger.info("LTE: Cannot get to online state {}".format(self.wan_state))
                elif self.wan_state in {WAN_STATE_LTE_ON, WAN_STATE_LTE_ON_ETH_OFF, WAN_STATE_LTE_ON_ETH_ON}:
                    logger.info("LTE: Goes offline (previously online)")

                lte_status = LTE_OFFLINE
            else:
                self.fail_counterA = self.fail_counterA + 1
        else:  # Ping was OK
            self.fail_counterA = 0
            self.rescue_counter = 0
            if self.wan_state in {WAN_STATE_LTE_OFF, WAN_STATE_LTE_OFF_ETH_OFF, WAN_STATE_LTE_OFF_ETH_ON}:
                logger.info("LTE: Goes online (previously offline) {}".format(self.wan_state))
            lte_status = LTE_OK
            logger.debug("LTE Ping OK")

        if current_config['ping']['autoswitch'] :  # Autoswitch feature is active and LTE status is resolved
            self.__autoswitch(lte_status)
        else:  # Autoswitch disabled, only backup iface periodical probing is active
            if lte_status == LTE_OK:
                self.wan_state = WAN_STATE_LTE_ON
            elif lte_status == LTE_OFFLINE:
                self.wan_state = WAN_STATE_LTE_OFF

        logger.debug("Current WAN state {}".format(self.wan_state))


def do_action(modem, action, errcode=ERROR_CODE_UNSPEC):  # Perform desired action (as a reaction to different error conditions)

    reinit = False

    if action == ACTION_SOFTRESTART :
        logger.info("Modem soft-restart")
        try:
            modem.restart(soft=True)
        except:
            logger.info("Modem soft-restart failed, performing HARD-restart")
            do_action(modem, ACTION_HARDRESTART)  # Hard restart as a fallback to the SOFT one (via AT command)

        reinit = True  # Re-init modem

    elif action == ACTION_HARDRESTART :
        if modem:
            del modem
        logger.info("Modem hard-restart caused by an error")
        LTEModem.restart_hw()  #
        reinit = True  # Re-init modem

    elif action == ACTION_SUSPEND :
        logger.warning("Daemon suspended...exit in {} sec".format(current_config['suspend_timeout']))
        if current_config['suspend_timeout'] < 0:
            do_action(modem, ACTION_HALT, errcode)
        if modem:
            modem.go_halt_suspend(errcode)
        time.sleep(current_config['suspend_timeout'])
        reinit = True  # Re-init modem

    elif action == ACTION_HALT :
        logger.warning("Daemon halted forever...must be restarted manually {}".format(errcode))
        if modem:
            modem.go_halt_suspend(errcode)
        while True :
            time.sleep(current_config['suspend_timeout'])

    return reinit


class LedServer(object):

    def __init__(self):  # Check presence of all leds
        ret = True
        ret &= os.path.exists(LED_INT_PATH)
        ret &= os.path.exists(LED_NET_PATH)
        ret &= os.path.exists(LED_SIG_PATH)
        ret &= os.path.exists(LED_SIM_PATH)

        self.leds = {"sim": LED_SIM_PATH,
              "net": LED_NET_PATH,
              "sig": LED_SIG_PATH,
              "int": LED_INT_PATH}

        self.led_states = {"off" : "0",
            "on" : "1",
            "blink_fast": "150",
            "blink_slow": "2000"}

        self.led_cache = {}
        for led in self.leds:  # Init da cache
            self.led_cache[led] = 0
            self.led_cache[led + '_duty'] = 0

        if not ret:
            raise LEDErrorException

        self.all_off()

    def set(self, led, state):  # Set state of LTE coupled LEDs

        if self.led_cache[led] == state:
            return

        state = state.lower()
        if state == "on" or state == "off" :
            value = open(self.leds[led] + "/trigger", "w")
            value.write(str("none"))
            value.close()

            value = open(self.leds[led] + "/brightness", "w")
            value.write(str(self.led_states[state]))
            value.close()
        else:  # Blink
            value = open(self.leds[led] + "/trigger", "w")
            value.write(str("timer"))
            value.close()

            value = open(self.leds[led] + "/delay_on", "w")
            value.write(str(self.led_states[state]))
            value.close()

            value = open(self.leds[led] + "/delay_off", "w")
            value.write(str(self.led_states[state]))
            value.close()

        self.led_cache[led] = state

    def duty(self, led, duty):  # Set LED duty cycle (used as a signal quality indicator)
        if self.led_cache[led + '_duty'] == duty and self.led_cache[led] == "duty" :
            return

        per = (3000 / 31) * (31 - duty)
        t_on = per  # 0-930
        t_off = per

        value = open(self.leds[led] + "/trigger", "w")
        value.write(str("timer"))
        value.close()

        value = open(self.leds[led] + "/delay_on", "w")
        value.write(str(t_on))
        value.close()

        value = open(self.leds[led] + "/delay_off", "w")
        value.write(str(t_off))
        value.close()

        self.led_cache[led + '_duty'] = duty
        self.led_cache[led] = "duty"

    def all_off(self):  # Switch of all LEDs
        for led in self.leds:
            self.set(led, 'off')

    def all_blink(self, fast=True):
        mode = 'off'
        for led in self.leds:
            self.set(led, mode)
        mode = 'blink_fast' if fast else 'blink_slow'
        for led in self.leds:
            self.set(led, mode)


class LTEModem(object):

    rept_rst_cnt = 0
    last_rst_time = time.time()

    REGEX_CSQ = re.compile(r"\+CSQ:\s*(\d+),(\d+)")
    REGEX_COP = re.compile(r"\+COPS:\s*\d+,\d?,\"(.*)\",(\d?)")

    def __init__(self, port, retry_counter=3):
        exc = None
        self.leds = None

        while retry_counter:
            try:
                self.iface = None
                if not self.leds:
                    self.leds = LedServer()
                self.__iface_init(port)
                return
            except Exception as E:
                retry_counter = retry_counter - 1
                logger.info("Modem init retry...{} attempts left".format(retry_counter))
                sleep(random.random() * 10 + 5)
                exc = E
                continue
        if self.leds:
            self.leds.all_blink(fast=False)
        raise ModemCreateInstanceException(str(exc))

    def __del__(self):
        print("Modem ending...")
        if self.iface:
            print("Closing serial line...")
            self.iface.close()

        if self.leds:
            self.leds.all_off()

    def __iface_init(self, port):

        if not stat.S_ISCHR(os.stat(port).st_mode):  # Check if modem virtual serial interface exists
            raise ModemErrorException("Modem serial port {} is not a valid character device".format(port))

        if hasattr(self.iface, 'open'):
            self.iface.close()

        self.iface = serial.Serial(port, 115200, timeout=0.3, parity=serial.PARITY_NONE, rtscts=0)

        current_config['lte_iface'] = get_lte_iface_name()
        if not current_config['lte_iface']:
            raise ModemErrorException("No network interface found on USB port {}".format(USB_PORT))

    def init(self):
        self.iface.read(4065)  # Dummy read to flush the input buffer
        retry_counter = 5

        while retry_counter:
            try:
                ret = self.query("AT\r\n", 10)  # Check if modem responds to AT commands
                if "OK" in ret:
                    break
            except TimeoutException:
                retry_counter = retry_counter - 1
                logger.debug("No reply from modem, retries left {}...".format(retry_counter))
                time.sleep(0.5)
                continue
            raise TimeoutException

        while "OK" not in self.query("AT+CMEE=1\r\n", 17):  # Set extended error report (see CME codes in manual)
            time.sleep(2)

        while True:  # Manage pin stuff
            chars = self.query("AT+CPIN?\r\n")
            if("SIM PIN" in chars):
                if(current_config['pin'] != None):
                    self.query("AT+CPIN={}\r\n".format(current_config['pin']))
                else: # Prevent to inserting invalid PIN if undefined in config file
                    logger.debug("No PIN code defined in {}".format(CONFILE_PATH))
                    raise ModemErrorException("ERROR: 16") #This is the same ERR code as generated by the LTE modem.
                continue
            elif("READY" in chars):
                logger.info("Pin resolved")
                self.leds.set('sim', 'on')
                break
            else:
                logger.debug("UNEXPECTED DATA RECEIVED len({}) {}".format(len(chars), chars))

        chars = self.query("AT+UUSBCONF?\r\n")  # Set appropriate USB mode (if needed)
        if(("0x1143" in chars and current_config['usbmode'] == "2") or
           ("0x1146" in chars and current_config['usbmode'] == "3")):
            logger.debug("USB profile remains the same")
        else:
            logger.info("Changing USB mode to " + str(current_config['usbmode']))
            self.query("AT+UUSBCONF = {}\r\n".format(current_config['usbmode']))
            raise NewSettinngs  # Restart is needed

        while True:  # Set APN
            chars = self.query("AT+UCGDFLT?\r\n")
            if(current_config['apn'] in chars):
                logger.debug("APN remains the same")
                break
            else:
                logger.info("Changing APN to " + current_config['apn'])
                self.iface.write("AT+UCGDFLT=1,{0}\r\n".format(current_config['apn']))
                raise NewSettinngs  # Restart is needed

        os.system("ip link set {} up".format(current_config['lte_iface']))
        logger.info("Modem init complete")

    def query(self, query, bytes_expected=569):
        chars = ""
        retry_cnt = 3

        while retry_cnt > 0:
            try:
                self.iface.flushOutput()  # Flush da buffer
                self.iface.flushInput()  # Flush da buffer
    	           #time.sleep(0.1)
                self.iface.write(query)  # Write op. is blocking - may take longer !
                chars = self.iface.read(bytes_expected)  # Read with previously defined timeout
            except Exception as e:  # May occur when other app uses modem's serial line
                raise ModemErrorException("Modem I/O error " + str(e))
            # logger.debug("RECEIVED {} bytes: {}".format(len(chars),chars))
            if "ERROR" in chars:
                raise ModemErrorException(chars)
            if len(chars) > 0: # No reply received
                return chars

            retry_cnt = retry_cnt - 1
            logger.info("More attempts for " + str(query))

        raise TimeoutException

    def query_extended_timeout(self, query, timeout, bytes_expected=569):

        ret = ""
        timeout_bckp = self.iface.timeout
        self.iface.timeout = timeout
        try:
            ret = self.query(query, bytes_expected)
        finally:
            self.iface.timeout = timeout_bckp
        return ret

    @staticmethod
    def restart_hw():  # Hardware restart by RST pin toggling. Static function - not tied with any instance.
        global logger

        try:
            ret = os.path.exists('/sys/class/gpio/gpio' + str(18))
            if ret == False :  # If not exported yet...
                with open('/sys/class/gpio/export', 'w') as f:  # Export it !
                    f.write(str(18))
                    f.close()

            with open('/sys/class/gpio/gpio' + str(18) + '/direction', 'w') as f:
                f.write('out')  # Set GPIO as output
                f.close()

            with open('/sys/class/gpio/gpio' + str(18) + '/value', 'w') as f:
                if logger:
                    logger.debug("Reseting LTE stage 1")
                else:
                    print("Restarting LTE stage 1")
                f.write("1")  # There is an inverting transistor before the RESET_N pin of the modem !
                f.flush()
                time.sleep(3)  # This time should be 2.3 - 15 sec according the datasheet
                if logger:
                    logger.debug("Reseting LTE stage 2")
                else:
                    print("Restarting LTE stage 2")
                f.write("0")  # Back to LOG 1
                f.flush()
                f.close()

        except BaseException as E:
            # logger.error("MODEM HW RESET ERROR: {}".format(str(E)))
            if logger :
                logger.error("Hard reset failed - {}".format(str(E)))
            else:
                print "Exc: " + str(E)

    def restart(self, soft=True):

        if soft == True:  # SW restart

            self.leds.all_blink()

            chars = self.query_extended_timeout("AT+CFUN=16\r\n", 3.0, 15)
            self.iface.close()

        else:  # HW restart
            try:
                LTEModem.restart_hw()
            except BaseException, E:
                logger.error("MODEM HW RESET ERROR: {}".format(str(E)))
                print "Exc: " + str(E)
                return

        while True:
            logger.debug("Waiting for modem disconnect...")
            try:
                stat.S_ISCHR(os.stat(MODEM_PORT).st_mode)
            except:
                break
            time.sleep(0.3)

        logger.debug("Disconnected")

        while True:
            logger.debug("Waiting for modem reconnect...")
            try:
                stat.S_ISCHR(os.stat(MODEM_PORT).st_mode)
            except:
                time.sleep(0.3)
                continue
            break

        logger.debug("Reconnected")

        self.iface.open()
        self.iface.flush()
        self.iface.reset_output_buffer()
        self.iface.reset_input_buffer()
        while True:
            self.iface.write("AT\r\n")
            if(len(chars) > 0):
                break

        logger.debug("RESTART complete ".format(len(chars), chars))

        self.leds.all_off()

        time.sleep(random.random() * 10 + 5.0)

        if not current_config['lte_iface']:
            current_config['lte_iface'] = get_lte_iface_name()

    def resolve_error(self, exc):

        action = ACTION_NOTHING
        errcode = ERROR_CODE_UNSPEC
        logger = logging.getLogger('unipilte')
        errstring = str(exc)

        if isinstance(exc, TimeoutException):
            logger.error("Modem reply timeout")
            action = ACTION_HARDRESTART
        elif "ERROR: 10" in errstring:
            logger.error("SIM Not inserted")
            self.leds.set('sim', 'blink_slow')
            errcode = ERROR_CODE_NO_SIM
            action = ACTION_SOFTRESTART
        elif "ERROR: 13" in errstring:
            logger.error("SIM Failure")
            self.leds.set('sim', 'blink_slow')
            errcode = ERROR_CODE_SIM_ERROR
            action = ACTION_SOFTRESTART
        elif "ERROR: 16" in errstring:
            logger.error("Incorrect or undefined password (PIN). Going to sleep...")
            self.leds.set('sim', 'blink_fast')
            errcode = ERROR_CODE_WRONG_PIN
            action = ACTION_HALT
        elif "Modem I/O error" in errstring:
            logger.error("Modem I/O error, performing HARD restart")
            action = ACTION_HARDRESTART
        elif "ERROR: 569" in errstring:
            logger.error("An error with code 569")
            action = ACTION_NOTHING
        else:
            logger.error("Unhandler error occured: " + errstring)
            action = ACTION_SOFTRESTART

        # Time filter - if more than THRESHOLD restarts occurs with minimal delay, daemon goes to SUSPEND
        if (action == ACTION_HARDRESTART or action == ACTION_SOFTRESTART):
            if (time.time() - LTEModem.last_rst_time) < 60.0:
                logger.info("Too rapid restart request, {} left to SUSPEND ".format(10 - LTEModem.rept_rst_cnt))
                LTEModem.rept_rst_cnt = LTEModem.rept_rst_cnt + 1

            if LTEModem.rept_rst_cnt > 10:
                logger.warn("Maximum number of resets reached, going to SUSPEND".format())
                LTEModem.rept_rst_cnt = 0
                LTEModem.last_rst_time = 0
                action = ACTION_SUSPEND

            LTEModem.last_rst_time = time.time()

        return action, errcode

    def update_led(self, is_registered, rssi, wan_state):

        if is_registered is True:  # Registered in the network
            self.leds.set('net', 'on')
        else:
            self.leds.set('net', 'off')

        if rssi == 99:
            self.leds.set('sig', 'off')
        else:
            self.leds.duty('sig', rssi)


        if current_config['ping']['autoswitch'] :
            if wan_state == WAN_STATE_LTE_ON_ETH_ON:  # Connection to the internet (or other WAN network)
                self.leds.set('int', 'on')
            elif wan_state in {WAN_STATE_LTE_ON_ETH_OFF, WAN_STATE_LTE_ON}:
                self.leds.set('int', 'blink_fast')
            elif wan_state in {WAN_STATE_LTE_OFF_ETH_ON, WAN_STATE_ETH_ON}:
                self.leds.set('int', 'blink_slow')
            else:  # OFFLINE
                self.leds.set('int', 'off')
        else: # No autoswitch
            if wan_state in {WAN_STATE_ETH_ON, WAN_STATE_LTE_ON}:
                self.leds.set('int', 'on')
            else:
                self.leds.set('int', 'off')


    def go_halt_suspend(self, errcode=ERROR_CODE_UNSPEC):

        if errcode in (ERROR_CODE_NO_SIM, ERROR_CODE_SIM_ERROR):
            self.leds.all_off()
            self.leds.set('sim', 'blink_slow')
        elif errcode == ERROR_CODE_WRONG_PIN:
            self.leds.all_off()
            self.leds.set('sim', 'blink_fast')
        else:
            self.leds.all_blink(fast=False)

    def get_operator_nettype(self):
        ret = self.query("AT+COPS?\r\n")
        res = self.REGEX_COP.search(ret)
        if res is None or len(res.groups()) != 2:
            #logger.info("Cannot parse operator info, received: {}".format(ret))
            return (None, None)

        return (res.group(1), res.group(2))


    def set_local_nic(ip, mask, dhcp_range_start, dhcp_range_end):
        ret = self.query("AT+UIPCONF=\"{}\",\"{}\",\"{}\",\"{}\"\r\n".ip,
                          mask, dhcp_range_start, dhcp_range_end)

    def get_sig_metrics(self):
        ret = self.query("AT+CSQ\r\n")
        res = self.REGEX_CSQ.search(ret)
        if res is None or len(res.groups()) != 2:
            logger.info("Cannot parse signal quality, received: {}".format(ret))
            return (99, 0)

        return (int(res.group(1)), int(res.group(2)))

#--------------------------------------------------------------------------
#--------------------------------------------------------------------------
#--------------------------------------------------------------------------


parser = argparse.ArgumentParser(description="2G/3G/LTE connection management daemon", epilog="unipi.technology 2019")
parser.add_argument("-r", "--reset", dest="reset", action="store_true", help="perform HW reset of the LTE module")
parser.add_argument("-d", "--delete", dest="delete", action="store_true", help="delete NIC from " + NIC_CONF_PATH)
args = parser.parse_args()

if args.reset :  # Single shot reset (call the script with -r or --reset)
    LTEModem.restart_hw()
    sys.exit(0)
elif args.delete :  # Single shot reset (call the script with -r or --reset)
    remove_iface_confile()
    sys.exit(0)

if os.path.isfile(pidfile):
    print "LTE Daemon already running, exiting..."
    sys.exit(0)

try:  # Daemon mode starts here

    pid = str(os.getpid())
    file(pidfile, 'w').write(pid)

    signal.signal(signal.SIGINT, sig_handler)  # LOG exit cause by signal in handler
    signal.signal(signal.SIGTERM, sig_handler)
    signal.signal(signal.SIGQUIT, sig_handler)

    conn_quard = None
    config = ConfigParser.ConfigParser()
    err_msg = ""

    if not os.path.isfile(CONFILE_PATH):  # Check the config file
      err_msg = "Config file not found on path: {}".format(CONFILE_PATH)

    try:
      config.read([CONFILE_PATH])
    except Exception as e :
      err_msg = "Error while parsing file {} : {}".format(CONFILE_PATH, str(e))

    log_path = getFromConfile(config, "LOG", "path", DEFAULT_LOG_PATH)
    log_level = getFromConfile(config, "LOG", "severity", DEFAULT_LOGGING_LEVEL).upper()
    log_size = getFromConfile(config, "LOG", "logfile_size", DEFAULT_LOGFILE_SIZE, integer=True)

    logger = logging.getLogger('unipilte')  # Global instance of the logger for this application

    try:
        logger.setLevel(log_level)
    except:
        logger.setLevel(DEFAULT_LOGGING_LEVEL)
        err_msg = "Invalid log severity"

    if not log_path:
        filelog_handler = logging.handlers.SysLogHandler(address='/dev/log')  # Standard rotated log
        log_formatter = logging.Formatter('%(name)s - %(levelname)s - %(message)s')
    else:
        filelog_handler = logging.handlers.RotatingFileHandler(filename=log_path, maxBytes=log_size, backupCount=1)  # Standard rotated log
        log_formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')

    filelog_handler.setFormatter(log_formatter)

    logger.addHandler(filelog_handler)
    logger.info("unipi-lte version {} - Logging started..".format(DAEMON_VERSION))

    if err_msg:  # Handle log messages from config file parsing stage
        logger.warn(err_msg + "; USING DEFAULT VALUES")
    else:
        logger.debug("Config file {} parsed".format(CONFILE_PATH))

    current_config = {}
    current_config['apn'] = getFromConfile(config, "MAIN", "apn", None)

    if not current_config['apn']:
        leds = LedServer()
        leds.all_off()
        lte_iface = get_lte_iface_name()
        os.system("ip link set {} down".format(lte_iface))
        logger.warn("No APN defined in {}, switching off iface {} and stopping unipi-lte service...".format(CONFILE_PATH, lte_iface))
        sys.exit(0)

    current_config['pin'] = getFromConfile(config, "MAIN", "pin", None)
    current_config['usbmode'] = getFromConfile(config, "MAIN", "usbmode", "2")
    current_config['report_period'] = getFromConfile(config, "MAIN", "report_period", 86400, integer=True)
    current_config['oneshot'] = getFromConfile(config, "MAIN", "oneshot", default=False, boolean=True)
    current_config['regtimeout'] = getFromConfile(config, "MAIN", "reg_timeout", 120, integer=True)
    current_config['suspend_timeout'] = getFromConfile(config, "MAIN", "suspend_timeout", 86400, integer=True)
    current_config['lte_autokeep_on'] = getFromConfile(config, "MAIN", "auto_keepon", default=True, boolean=True)

    current_config['lte_iface'] = None

    current_config['ping'] = {}
    current_config['ping']['active'] = getFromConfile(config, "PING", "active", default=False, boolean=True)

    if current_config['ping']['active'] :
        current_config['ping']['timeout'] = int(getFromConfile(config, "PING", "timeout", 60))
        current_config['ping']['period'] = int(getFromConfile(config, "PING", "period", 20))
        current_config['ping']['threshold'] = int(getFromConfile(config, "PING", "fail_threshold", 3))
        current_config['ping']['ip_lte'] = getFromConfile(config, "PING", "ip_lte", "8.8.8.8")
        current_config['ping']['autoswitch'] = getFromConfile(config, "PING", "autoswitch", default=False, boolean=True)
        current_config['ping']['eth_iface'] = getFromConfile(config, "PING", "primary_iface", "eth0")

        if current_config['ping']['autoswitch'] :
            current_config['ping']['ip_eth'] = getFromConfile(config, "PING", "ip_eth", "8.8.4.4")

        conn_quard = ConnectionGuard()

    logger.debug("CURRENT SETTINGS: {} ".format(str(current_config)))

    # Regex parsing some specific AT commands replies

    REGEX_IP = re.compile(r"\+UIPADDR:\s*\d+,\"\w*\",\"([^\"]*)\"")
    REGEX_PRIV_IP = re.compile(r"\+UIPCONF:\s\"([^\"]*)\",\"([^\"]*)\"")

    modem = None
    stats = Stats(current_config['report_period'], current_config['ping']['active'])
    logger.debug("Entering the main loop...")


    ping_timestep = 0
    reg_timeout = current_config['regtimeout']
    is_registered = False
    start_timestamp = 0


    reinit = True  # Init after start

    while True:
        try:
            start_timestamp = time.time()
            if reinit == True:
                reinit = False
                wan_available = False
                # if not modem: # Try to create modem instance - working serial port needed
                if modem:
                    del modem
                modem = LTEModem(MODEM_PORT)
                modem.init()
                #--------- TODO - udelat jinak, je to totiz stejne jako na zacatku -----------
                # !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
                if conn_quard is not None:
                    conn_quard.reset_state()

                ping_timestep = 0
                reg_timeout = current_config['regtimeout']
                is_registered = False
                start_timestamp = 0

                #------------------------------------------------------------------------------

                if current_config['oneshot'] :
                    logger.info("One-shot mode, exiting...")
                    sys.exit(0)

            rssi, sigqual = modem.get_sig_metrics()
            operator, nettype = modem.get_operator_nettype()
            if operator is not None:  # Registered to the network, thus nettype is valid

                reg_timeout = current_config['regtimeout'] # Reset non-registered counter

                if is_registered is False:  # Registered for a first time
                    is_registered = True
                    logger.info("Registered to: {} Network type: {} RSSI: {} Sigqual: {}".format(operator, nettype, rssi, sigqual))

                    # Check local IP address assigned by modems DHCP server
                    ip = REGEX_IP.search(modem.query_extended_timeout("AT+UIPADDR=\r\n", timeout=3.0))
                    priv_ip = REGEX_PRIV_IP.search(modem.query_extended_timeout("AT+UIPCONF?\r\n", timeout=3.0))

                    if ip is not None:  # Toto je potreba osetrit vsude - neny IP, neny NET !!!!!!!!
                        new_net_settings = resolve_subnets(ip.group(1), priv_ip.group(1), priv_ip.group(2))
                        if new_net_settings:
                            modem.set_local_nic(new_net_settings['ip'], new_net_settings['mask'], new_net_settings['dhcp'], new_net_settings['dhcp'])
                            raise NewSettinngs
                    else:
                        logger.warning("Cannot get WAN IP adress from the module")

            else: # Not registered to operator
                if is_registered is True:
                    logger.info("Deregistered from network operator. {} Cycles to restart".format(reg_timeout))
                is_registered = False
                reg_timeout -= 1
                if reg_timeout == 0 :
                    reg_timeout = current_config['regtimeout']
                    raise TimeoutException("Cannot register to network")
                modem.query("AT+CPIN?\r\n")  # Check da SIM

            logger.debug("RSSI: {0} sigqual: {1} net-type: {2}".format(rssi, sigqual, nettype))

            if not current_config['ping']['active'] or not operator:  # Manage ping
                pass
            elif ping_timestep:
                ping_timestep -= 1
                logger.debug("Next ping in " + str(ping_timestep) + " cycles")
            else:
                ping_timestep = current_config['ping']['period']
                conn_quard.ping()
                if not conn_quard.get_rtable_valid():
                    ping_timestep = 0  # If rtable is invalid, try ping again a bit earlier
                stats.report_metrics(rssi, sigqual, conn_quard.get_wan_status(), nettype)

            if stats.is_time_fullfilled():  # Generate report to the log
                logger.info(stats.get_summmary())
                stats.reset()

            stats.report_partial(rssi, sigqual)
            modem.update_led(is_registered, rssi, conn_quard.get_wan_status())
            # modem.update_led(operator, rssi, wan_available, conn_quard.is_backup_active())
            time.sleep(10 - ((time.time() - start_timestamp) % 10))
            logger.debug("STEP")

        except ModemCreateInstanceException as e:
            logger.error("Cannot create modem instance: " + str(e))
            do_action(None, ACTION_SUSPEND)
            reinit = do_action(None, ACTION_HARDRESTART)
            continue

        except ConnectionError as e:
            logger.error("Cannot connect to WAN: " + str(e) + " restarting...")
            reinit = do_action(None, ACTION_HARDRESTART)
            continue

        except (ModemErrorException, TimeoutException) as e:
            action, errcode = modem.resolve_error(e)
            reinit = do_action(modem, action, errcode)
            logger.debug("Continuing")
            continue

        except NewSettinngs as e:
            logger.info("Settings of the modem has been changed, restarting...")
            reinit = do_action(modem, ACTION_SOFTRESTART)
            continue

        except(KeyboardInterrupt):
            led.close()
            logging.shutdown()
            print("Good bye")

except Exception as E:
    print(str(E))
    pass

finally:
    os.unlink(pidfile)
